-- ============================================================
-- Proyecto: 	Laboratorio PS2 DL
-- Modulo:		Receptor PS2
-- Objetivo:	Pasar de serie a paralelo la tecla oprimida
-- Nota:		
-- Modificado:	27-OCT-2015
-- ============================================================

---------------------------------------------------------
-- RECEPTOR PS2
---------------------------------------------------------
LIBRARY ieee ;
USE ieee.std_logic_1164.all;
use IEEE.numeric_std.all; 
USE ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

-----------------------------------------------------------
----- ENTITY RECEPTOR PS2
-----------------------------------------------------------


ENTITY Receptor_PS2 IS
   	PORT(
      DATA			: IN STD_LOGIC;
      CLK_PS2		: IN STD_LOGIC;
      RSTn    		: IN STD_LOGIC;
      CLK      		: IN STD_LOGIC;
	  Num			: OUT STD_LOGIC_VECTOR(3 DOWNTO 0);
	  Kv			: OUT std_logic
      );
END;



ARCHITECTURE BEHAV OF Receptor_PS2 IS
-- Seales y estados

TYPE state_type IS (s1,s2,s3,s4);
SIGNAL STATE, NXSTATE : state_type;
SIGNAL STATE_kv, NXSTATE_kv : state_type;
SIGNAL STATE_d, NXSTATE_d : state_type;
SIGNAL D : STD_LOGIC_VECTOR(7 DOWNTO 0);
SIGNAL R : STD_LOGIC;
SIGNAL C : STD_LOGIC;
SIGNAL F : STD_LOGIC;
SIGNAL CLK_PS2_SINC : STD_LOGIC;
SIGNAL DATA_SINC : STD_LOGIC;
SIGNAL CLK_PS2_PRE_SINC : STD_LOGIC;
SIGNAL DATA_PRE_SINC : STD_LOGIC;
SIGNAL NUM_OK : STD_LOGIC;
SIGNAL F0_OK : STD_LOGIC;
SIGNAL RDY : STD_LOGIC;
SIGNAL CONT : STD_LOGIC_VECTOR(3 DOWNTO 0);
      
BEGIN

-- Maquina de estados detector de flancos. La seal F se usa como enable de D y CONT
comb_maquina_detector: PROCESS (STATE_d,CLK_PS2_SINC)
BEGIN
 CASE STATE_d IS
  WHEN s1 =>
   IF (CLK_PS2_SINC='1')THEN NXSTATE_d <= s1; ELSE NXSTATE_d <= s3; END IF;
   F <= '0';
  WHEN s2 =>
   NXSTATE_d <= s1; 
   F <= '1';
  WHEN s3 =>
   IF (CLK_PS2_SINC='1')THEN NXSTATE_d <= s2; ELSE NXSTATE_d <= s3; END IF;
   F <= '0';
  WHEN OTHERS =>     
   NXSTATE_d <= s1;
   F <= '0';
  END CASE;
END PROCESS;

-- Process donde se generan los FF que guardan el dato
datos: PROCESS (CLK,DATA_SINC,F)
BEGIN
 IF CLK'EVENT AND CLK = '1' THEN
  IF F='1' THEN
   D(6 DOWNTO 0) <= D(7 DOWNTO 1);
   D(7) <= DATA_SINC;
  END IF;
 END IF;
END PROCESS;

-- Process donde se generan los FF del contador
contador: PROCESS (CLK,F,R)
BEGIN
 IF R='1' THEN
  IF CLK'EVENT and CLK = '1' THEN
   IF F='1' THEN CONT <= CONT+1; END IF;
  END IF;
 ELSE
  CONT <= (others =>'0');
 END IF;
END PROCESS;

-- Process donde se genera la seal c
combinatorio_c: PROCESS (CONT)
BEGIN
 IF CONT="1001" THEN C <= '1'; ELSE C <= '0'; END IF;
END PROCESS;

-- Maquina de estados control sap
camb_maquina_sap: PROCESS (STATE,C,DATA_SINC)
BEGIN
 CASE STATE IS
  WHEN s1 =>
   IF DATA_SINC ='1' THEN NXSTATE <= s1; ELSE NXSTATE <= s2; END IF;
   R <= '0';
   RDY <= '0';
  WHEN s2 =>
   IF C ='0' THEN NXSTATE <= s2; ELSE NXSTATE <= s3; END IF;
   R <= '1';
   RDY <= '0';
  WHEN s3 =>
   NXSTATE <= s4;
   R <= '1';
   RDY <= '1';
  WHEN s4 =>
   IF (DATA_SINC ='1' AND C = '0') THEN NXSTATE <= s1; ELSE	NXSTATE <= s4;END IF;
   R <= '1';
   RDY <= '0';
  WHEN OTHERS =>
   NXSTATE <= s1; 
   R <= '0';
   RDY <= '0';
  END CASE;
END PROCESS;
   
-- Maquina de estados valid
comb_maquina_valid: PROCESS (STATE_kv,NUM_OK,F0_OK, RDY)
BEGIN
 CASE STATE_kv IS
  WHEN s1 =>
   IF (RDY='1' AND F0_OK = '1')THEN NXSTATE_kv <= s2; ELSE NXSTATE_kv <= s1; END IF;
   Kv <= '0';
  WHEN s2 =>
   IF RDY ='0' THEN NXSTATE_kv <= s2;	ELSE NXSTATE_kv <= s1;	END IF;
   IF (NUM_OK = '1' AND RDY = '1') THEN Kv <= '1'; ELSE Kv<= '0'; END IF;
  WHEN OTHERS =>     
   NXSTATE_kv <= s1;
   Kv <= '0';
 END CASE;
END PROCESS;
   
-- combinarotia deco y numok y f0ok
combinatorio_ps2_a_num: PROCESS (D)
BEGIN
 CASE D IS
  WHEN X"70" =>	Num <= "0000";NUM_OK <='1';	F0_OK<='0'; 
  WHEN X"69" =>	Num <= "0001";NUM_OK <='1';	F0_OK<='0';
  WHEN X"72" =>	Num <= "0010";NUM_OK <='1';	F0_OK<='0';
  WHEN X"7A" =>	Num <= "0011";NUM_OK <='1';	F0_OK<='0';
  WHEN X"6B" => Num <= "0100";NUM_OK <='1'; F0_OK<='0';
  WHEN X"73" =>	Num <= "0101";NUM_OK <='1'; F0_OK<='0';		
  WHEN X"74" =>	Num <= "0110";NUM_OK <='1'; F0_OK<='0';
  WHEN X"6C" =>	Num <= "0111";NUM_OK <='1';	F0_OK<='0';
  WHEN X"75" =>	Num <= "1000";NUM_OK <='1';	F0_OK<='0';
  WHEN X"7D" => Num <= "1001";NUM_OK <='1'; F0_OK<='0';
  WHEN X"F0" =>	Num <= "XXXX";NUM_OK <='0'; F0_OK<='1';
  WHEN OTHERS =>Num <= "XXXX";NUM_OK <='0'; F0_OK<='0';
 END CASE;
END PROCESS;	
   
   
-- Process donde se generan los FF
-- sensibles a flanco
-- de subida de la mquina de estado
sync:PROCESS (CLK,RSTn,STATE)
BEGIN
 IF RSTn = '0' THEN
  STATE <= s1;
  STATE_kv <= s1;
  STATE_d <= s1;
  CLK_PS2_PRE_SINC<='1';
  DATA_PRE_SINC<='1';
 ELSIF CLK'EVENT AND CLK = '1' THEN
  STATE <= NXSTATE;
  STATE_kv <= NXSTATE_kv;
  STATE_d <= NXSTATE_d;
  CLK_PS2_PRE_SINC<=CLK_PS2;
  DATA_PRE_SINC<=DATA;
  CLK_PS2_SINC<=CLK_PS2_PRE_SINC;
  DATA_SINC<=DATA_PRE_SINC;
 END IF;
END PROCESS;

END BEHAV;